package com.ho.test.perf.measure.annotations;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

/**
 * AspectPerformanceWatcher : 
 * + AspectJ applying on the elements annotated by the annotation PerformanceWatcher to check 
 * 	if the execution time of a method for this element exceeds the specified admissible threshold (warning duration).
 * 
 * + AspectJ allows the calculation of the execution time of a method and add a trace in the logs via mhode "calculatePerformance".
 * @author huseyin
 *
 */
@Component
@Aspect
@Order(5)
public class AspectPerformanceWatcher {

	private static Log log = LogFactory.getLog(AspectPerformanceWatcher.class);
		
	@SuppressWarnings("unchecked")
	@Around("within(@com.ho.test.perf.measure.annotations.PerformanceWatcher *)")
	public Object calculatePerformance(ProceedingJoinPoint pjp) throws Throwable{
		final long start = System.currentTimeMillis();
			
		// Execution of target method
		final Object result = pjp.proceed();
			
		final long duration = System.currentTimeMillis() - start;
			
		// Get the definition of annotation and of this admissible threshold (warning duration)
		final PerformanceWatcher performanceWatcher = (PerformanceWatcher) pjp.getSourceLocation().getWithinType().getAnnotation(PerformanceWatcher.class);
			
		// Check if the method's execution time exceeds the admissible threshold
		if(performanceWatcher != null && performanceWatcher.msDurationWarning() < duration){
				log.warn("**** THE METHOD "+pjp.getSignature().getName()+" FROM "+pjp.getSignature().getDeclaringTypeName()+" TOOK "+duration+ " MS (max"+performanceWatcher.msDurationWarning()+"ms). ****");
		}//end-if
		
		return result;
	}
}
